==> mule-message-metadata-model-1.5.0-rc3-sources.jar: org/mule/metadata/message/api/DefaultMessageMetadataType.java <==
/*
 * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
 * The software in this package is published under the terms of the CPAL v1.0
 * license, a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 */
package org.mule.metadata.message.api;


import org.mule.metadata.api.annotation.TypeAnnotation;
import org.mule.metadata.api.builder.ObjectFieldTypeBuilder;
import org.mule.metadata.api.model.MetadataFormat;
import org.mule.metadata.api.model.MetadataType;
import org.mule.metadata.api.model.ObjectFieldType;
import org.mule.metadata.api.model.ObjectType;
import org.mule.metadata.api.model.impl.BaseMetadataType;
import org.mule.metadata.api.visitor.MetadataTypeVisitor;

import java.lang.reflect.Field;
import java.util.Collection;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.function.Consumer;
import java.util.stream.Collectors;

import static java.util.stream.Stream.of;
import static org.apache.commons.lang3.ArrayUtils.addAll;

/**
 * Default implementation for {@link MessageMetadataType}.
 *
 * @since 1.0
 */
class DefaultMessageMetadataType extends BaseMetadataType implements ObjectType, MessageMetadataType {

  public static final String PAYLOAD_FIELD_NAME = "payload";
  public static final String ATTRIBUTES_FIELD_NAME = "attributes";
  private final Optional<ObjectFieldType> payloadMetadataType;
  private final Optional<ObjectFieldType> attributes;


  DefaultMessageMetadataType(Optional<MetadataType> payloadMetadataType, Optional<MetadataType> attributes,
                             Map<Class<? extends TypeAnnotation>, TypeAnnotation> annotations) {
    this(payloadMetadataType, null, attributes, null, annotations);
  }

  DefaultMessageMetadataType(Optional<MetadataType> payloadMetadataType,
                             Consumer<ObjectFieldTypeBuilder> payloadObjectFieldTypeBuilderConsumer,
                             Optional<MetadataType> attributes,
                             Consumer<ObjectFieldTypeBuilder> attributesObjectFieldTypeBuilderConsumer,
                             Map<Class<? extends TypeAnnotation>, TypeAnnotation> annotations) {
    super(MetadataFormat.JAVA, annotations);
    this.payloadMetadataType = payloadMetadataType.map((type) -> {
      final ObjectFieldTypeBuilder objectFieldTypeBuilder = new ObjectFieldTypeBuilder(MetadataFormat.JAVA);
      objectFieldTypeBuilder.key(PAYLOAD_FIELD_NAME).required(true).value(type);
      Optional.ofNullable(payloadObjectFieldTypeBuilderConsumer)
          .ifPresent(objectFieldTypeBuilderConsumer -> payloadObjectFieldTypeBuilderConsumer.accept(objectFieldTypeBuilder));
      return objectFieldTypeBuilder.build();
    });

    this.attributes = attributes.map((type) -> {
      final ObjectFieldTypeBuilder objectFieldTypeBuilder = new ObjectFieldTypeBuilder(MetadataFormat.JAVA);
      objectFieldTypeBuilder.key(ATTRIBUTES_FIELD_NAME).required(true).value(type);
      Optional.ofNullable(payloadObjectFieldTypeBuilderConsumer)
          .ifPresent(objectFieldTypeBuilderConsumer -> attributesObjectFieldTypeBuilderConsumer.accept(objectFieldTypeBuilder));
      return objectFieldTypeBuilder.build();
    });
  }

  @Override
  public Optional<MetadataType> getOpenRestriction() {
    return Optional.empty();
  }

  @Override
  public Collection<ObjectFieldType> getFields() {
    return of(payloadMetadataType, attributes)
        .filter((t) -> t.isPresent())
        .map((t) -> t.get())
        .collect(Collectors.toList());
  }

  @Override
  public boolean isOrdered() {
    return false;
  }

  @Override
  public Optional<ObjectFieldType> getFieldByName(String name) {
    switch (name) {
      case PAYLOAD_FIELD_NAME:
        return payloadMetadataType;
      case ATTRIBUTES_FIELD_NAME:
        return attributes;
    }
    return Optional.empty();
  }

  @Override
  public void accept(MetadataTypeVisitor metadataTypeVisitor) {
    metadataTypeVisitor.visitObject(this);
  }

  @Override
  public Optional<MetadataType> getPayloadType() {
    return payloadMetadataType.map(ObjectFieldType::getValue);
  }

  @Override
  public Optional<Set<TypeAnnotation>> getPayloadAnnotations() {
    return payloadMetadataType.map(MetadataType::getAnnotations);
  }

  @Override
  public <T extends TypeAnnotation> Optional<T> getPayloadAnnotation(Class<T> annotation) {
    return payloadMetadataType.flatMap((t) -> t.getAnnotation(annotation));
  }

  @Override
  public Optional<MetadataType> getAttributesType() {
    return attributes.map(ObjectFieldType::getValue);
  }

  @Override
  public Optional<Set<TypeAnnotation>> getAttributesAnnotations() {
    return attributes.map(MetadataType::getAnnotations);
  }

  @Override
  public <T extends TypeAnnotation> Optional<T> getAttributesAnnotation(Class<T> annotation) {
    return attributes.flatMap((t) -> t.getAnnotation(annotation));
  }

  private static final Field[] REFLECTION_FIELDS;
  static {
    try {
      REFLECTION_FIELDS = addAll(BASE_REFLECTION_FIELDS,
                                 DefaultMessageMetadataType.class.getDeclaredField("payloadMetadataType"),
                                 DefaultMessageMetadataType.class.getDeclaredField("attributes"));
    } catch (NoSuchFieldException e) {
      throw new RuntimeException("Could not resolve field", e);
    }
  }

  @Override
  public Field[] getReflectionComparableFields() {
    return REFLECTION_FIELDS;
  }
}

==> mule-message-metadata-model-1.6.16-20250218.213723-1-sources.jar: org/mule/metadata/message/api/DefaultMessageMetadataType.java <==
/*
 * Copyright 2023 Salesforce, Inc. All rights reserved.
 * The software in this package is published under the terms of the CPAL v1.0
 * license, a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 */
package org.mule.metadata.message.api;


import static java.util.Collections.addAll;
import static java.util.stream.Stream.of;

import org.mule.metadata.api.annotation.TypeAnnotation;
import org.mule.metadata.api.builder.ObjectFieldTypeBuilder;
import org.mule.metadata.api.model.MetadataFormat;
import org.mule.metadata.api.model.MetadataType;
import org.mule.metadata.api.model.ObjectFieldType;
import org.mule.metadata.api.model.ObjectType;
import org.mule.metadata.api.model.impl.BaseMetadataType;
import org.mule.metadata.api.visitor.MetadataTypeVisitor;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.function.Consumer;
import java.util.stream.Collectors;

/**
 * Default implementation for {@link MessageMetadataType}.
 *
 * @since 1.0
 */
class DefaultMessageMetadataType extends BaseMetadataType implements ObjectType, MessageMetadataType {

  public static final String PAYLOAD_FIELD_NAME = "payload";
  public static final String ATTRIBUTES_FIELD_NAME = "attributes";
  private final Optional<ObjectFieldType> payloadMetadataType;
  private final Optional<ObjectFieldType> attributes;
  private Object[] fieldValues;


  DefaultMessageMetadataType(Optional<MetadataType> payloadMetadataType, Optional<MetadataType> attributes,
                             Map<Class<? extends TypeAnnotation>, TypeAnnotation> annotations) {
    this(payloadMetadataType, null, attributes, null, annotations);
  }

  DefaultMessageMetadataType(Optional<MetadataType> payloadMetadataType,
                             Consumer<ObjectFieldTypeBuilder> payloadObjectFieldTypeBuilderConsumer,
                             Optional<MetadataType> attributes,
                             Consumer<ObjectFieldTypeBuilder> attributesObjectFieldTypeBuilderConsumer,
                             Map<Class<? extends TypeAnnotation>, TypeAnnotation> annotations) {
    super(MetadataFormat.JAVA, annotations);
    this.payloadMetadataType = payloadMetadataType.map((type) -> {
      final ObjectFieldTypeBuilder objectFieldTypeBuilder = new ObjectFieldTypeBuilder(MetadataFormat.JAVA);
      objectFieldTypeBuilder.key(PAYLOAD_FIELD_NAME).required(true).value(type);
      Optional.ofNullable(payloadObjectFieldTypeBuilderConsumer)
          .ifPresent(objectFieldTypeBuilderConsumer -> payloadObjectFieldTypeBuilderConsumer.accept(objectFieldTypeBuilder));
      return objectFieldTypeBuilder.build();
    });

    this.attributes = attributes.map((type) -> {
      final ObjectFieldTypeBuilder objectFieldTypeBuilder = new ObjectFieldTypeBuilder(MetadataFormat.JAVA);
      objectFieldTypeBuilder.key(ATTRIBUTES_FIELD_NAME).required(true).value(type);
      Optional.ofNullable(payloadObjectFieldTypeBuilderConsumer)
          .ifPresent(objectFieldTypeBuilderConsumer -> attributesObjectFieldTypeBuilderConsumer.accept(objectFieldTypeBuilder));
      return objectFieldTypeBuilder.build();
    });
  }

  @Override
  public Optional<MetadataType> getOpenRestriction() {
    return Optional.empty();
  }

  @Override
  public Collection<ObjectFieldType> getFields() {
    return of(payloadMetadataType, attributes)
        .filter((t) -> t.isPresent())
        .map((t) -> t.get())
        .collect(Collectors.toList());
  }

  @Override
  public boolean isOrdered() {
    return false;
  }

  @Override
  public Optional<ObjectFieldType> getFieldByName(String name) {
    switch (name) {
      case PAYLOAD_FIELD_NAME:
        return payloadMetadataType;
      case ATTRIBUTES_FIELD_NAME:
        return attributes;
    }
    return Optional.empty();
  }

  @Override
  public void accept(MetadataTypeVisitor metadataTypeVisitor) {
    metadataTypeVisitor.visitObject(this);
  }

  @Override
  public Optional<MetadataType> getPayloadType() {
    return payloadMetadataType.map(ObjectFieldType::getValue);
  }

  @Override
  public Optional<Set<TypeAnnotation>> getPayloadAnnotations() {
    return payloadMetadataType.map(MetadataType::getAnnotations);
  }

  @Override
  public <T extends TypeAnnotation> Optional<T> getPayloadAnnotation(Class<T> annotation) {
    return payloadMetadataType.flatMap((t) -> t.getAnnotation(annotation));
  }

  @Override
  public Optional<MetadataType> getAttributesType() {
    return attributes.map(ObjectFieldType::getValue);
  }

  @Override
  public Optional<Set<TypeAnnotation>> getAttributesAnnotations() {
    return attributes.map(MetadataType::getAnnotations);
  }

  @Override
  public <T extends TypeAnnotation> Optional<T> getAttributesAnnotation(Class<T> annotation) {
    return attributes.flatMap((t) -> t.getAnnotation(annotation));
  }

  public Object[] getFieldValues() {
    if (fieldValues == null) {
      fieldValues = createFieldValuesArray();
    }
    return fieldValues;
  }

  private Object[] createFieldValuesArray() {
    List<Object> fieldValues = new ArrayList<>();
    addAll(fieldValues, super.getFieldValues());
    addAll(fieldValues, payloadMetadataType, attributes);
    return fieldValues.toArray(new Object[fieldValues.size()]);
  }
}
